# Class: galera::server
#
# manages the installation of the galera server.
# manages the package, service, galera.cnf
#
# Parameters:
#  [*config_hash*]           - Hash of config parameters that need to be set.
#  [*enabled*]               - Defaults to true, boolean to set service ensure.
#  [*manage_service*]        - Boolean dictating if galera::server should manage the service.
#  [*bootstrap*]             - Defaults to false, boolean to set cluster boostrap.
#  [*package_ensure*]        - Ensure state for package. Can be specified as version.
#  [*package_name*]          - The name of the galera package.
#  [*service_name*]          - The name of the galera service.
#  [*service_provider*]      - What service provider to use.
#  [*wsrep_bind_address*]    - Address to bind galera service.
#  [*wsrep_provider*]        - Full path to wsrep provider library or 'none'.
#  [*wsrep_cluster_name*]    - Logical cluster name. Should be the same for all nodes.
#  [*wsrep_cluster_members*] - List of cluster members, IP addresses or hostnames.
#  [*wsrep_sst_method*]      - State snapshot transfer method.
#  [*wsrep_sst_username*]    - Username used by the wsrep_sst_auth authentication string.
#  [*wsrep_sst_password*]    - Password used by the wsrep_sst_auth authentication string.
#  [*wsrep_ssl_key*]         - A private key for the certificate above, unencrypted, in PEM format.
#  [*wsrep_ssl_cert*]        - Certificate file in PEM format.
#
# Actions:
#
# Requires:
#
# Sample Usage:
# class { 'galera::server':
#   config_hash => {
#     bind_address   => '0.0.0.0',
#     default_engine => 'InnoDB',
#     root_password  => 'root_pass',
#   },
#   wsrep_cluster_name => 'galera_cluster',
#   wsrep_sst_method   => 'rsync'
#   wsrep_sst_username => 'ChangeMe',
#   wsrep_sst_password => 'ChangeMe',
# }
#
class galera::server (
  $config_hash           = {},
  $enabled               = true,
  $manage_service        = true,
  $bootstrap             = false,
  $package_ensure        = 'present',
  $package_name          = 'mariadb-galera-server',
  $service_name          = $mysql::params::service_name,
  $service_provider      = $mysql::params::service_provider,
  $wsrep_bind_address    = '0.0.0.0',
  $wsrep_provider        = '/usr/lib64/galera/libgalera_smm.so',
  $wsrep_cluster_name    = 'galera_cluster',
  $wsrep_cluster_members = [ $::ipaddress ],
  $wsrep_sst_method      = 'rsync',
  $wsrep_sst_username    = 'root',
  $wsrep_sst_password    = undef,
  $wsrep_ssl_key         = undef,
  $wsrep_ssl_cert        = undef,
) inherits mysql {

  $config_class = { 'mysql::config' => $config_hash }

  create_resources( 'class', $config_class )

  package { 'galera':
    ensure => $package_ensure,
    name   => $package_name,
  }

  $opt_hash = delete_undef_values(
    {
      'socket.ssl_key'  => $wsrep_ssl_key,
      'socket.ssl_cert' => $wsrep_ssl_cert,
    })

  $wsrep_provider_options = join(join_keys_to_values($opt_hash, '='), "; ")

  file { '/etc/my.cnf.d/galera.cnf':
    ensure  => present,
    mode    => '0644',
    owner   => 'root',
    group   => 'root',
    content => template('galera/wsrep.cnf.erb'),
    require => File_line['includedir'],
    notify  => Service['galera'],
  }

  if $enabled {
    $service_ensure = 'running'
  } else {
    $service_ensure = 'stopped'
  }

  if $manage_service {
    Service['galera'] -> Exec<| title == 'set_mysql_rootpw' |>

    service { 'galera':
      ensure   => $service_ensure,
      name     => $service_name,
      enable   => $enabled,
      require  => Package['galera'],
      provider => $service_provider,
    }
  }

  file_line { 'includedir':
    path  => '/etc/my.cnf',
    line  => '!includedir /etc/my.cnf.d/',
    match => '^!includedir.*',
  }
}
